'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _flatten = require('lodash/flatten');

var _flatten2 = _interopRequireDefault(_flatten);

var _oors = require('oors');

var _PermissionsManager = require('./libs/PermissionsManager');

var _PermissionsManager2 = _interopRequireDefault(_PermissionsManager);

var _schemas = require('./schemas');

var schemas = _interopRequireWildcard(_schemas);

var _Security = require('./services/Security');

var _Security2 = _interopRequireDefault(_Security);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

class Security extends _oors.Module {
  constructor(...args) {
    var _temp;

    return _temp = super(...args), this.name = 'oors.security', this.hooks = {
      'oors.security.createPermissionsExtractor': ({ security }) => user => security.getUserPermissions({
        userId: user._id
      })
    }, _temp;
  }

  // eslint-disable-next-line class-methods-use-this
  buildExtractor(extractors) {
    return (() => {
      var _ref = _asyncToGenerator(function* (user) {
        const permissionsLists = yield Promise.all(extractors.filter(function (extractor) {
          return extractor;
        }).map(function (extractor) {
          return extractor(user);
        }));
        return (0, _flatten2.default)(permissionsLists);
      });

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    })();
  }

  setup() {
    var _this = this;

    return _asyncToGenerator(function* () {
      const [{ createRepository }, { registerServices }] = yield _this.dependencies(['oors.mongoDb', 'oors.octobus']);

      const services = registerServices(_this, {
        GroupRepository: createRepository({
          name: 'SecurityGroup',
          schema: schemas.group
        }),
        UserRepository: createRepository({
          name: 'SecurityUser',
          schema: schemas.user
        }),
        Security: new _Security2.default()
      });

      const extractors = yield _this.runHook('createPermissionsExtractor',
      // eslint-disable-next-line consistent-return
      function (module) {
        const permissionsExtractor = (0, _get2.default)(module, 'security.permissionsExtractor');
        if (permissionsExtractor) {
          return permissionsExtractor({
            security: services.Security
          });
        }
      }, {
        security: services.Security
      });

      const permissionsManager = new _PermissionsManager2.default({
        extractor: _this.buildExtractor(extractors)
      });

      yield _this.runHook('definePermissions', function (module) {
        const modulePermissions = (0, _get2.default)(module, 'security.permissions');
        if (modulePermissions) {
          Object.keys(modulePermissions).forEach(function (permission) {
            permissionsManager.define(permission, modulePermissions[permission]);
          });
        }
      }, {
        permissionsManager
      });

      _this.export(_extends({}, services, {
        permissionsManager
      }));
    })();
  }
}

Security.schema = {
  extractUserPermissions: _joi2.default.func()
};
exports.default = Security;