'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /* eslint-disable class-methods-use-this */


class Security {
  constructor({ UserRepository, GroupRepository }) {
    this.UserRepository = UserRepository;
    this.GroupRepository = GroupRepository;
  }

  setUserPermissions({ userId, permissions }) {
    var _this = this;

    return _asyncToGenerator(function* () {
      (0, _invariant2.default)(userId, 'User id is required!');
      (0, _invariant2.default)(permissions && Array.isArray(permissions), 'Permissions array is required!');

      const { UserRepository } = _this;

      const user = yield UserRepository.findOne({
        query: {
          userId
        }
      });

      return user ? UserRepository.updateOne({
        query: {
          userId
        },
        update: {
          $set: {
            permissions
          }
        }
      }) : UserRepository.createOne({
        userId,
        permissions
      });
    })();
  }

  getUserPermissions({ userId, includeGroups = false }) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      (0, _invariant2.default)(userId, 'User id is required!');

      const { UserRepository, GroupRepository } = _this2;

      const user = yield UserRepository.findOne({
        query: {
          userId
        }
      });

      let groupsPermissions = [];

      if (!!includeGroups && Array.isArray(user.groupIds) && user.groupIds.length) {
        const groups = yield GroupRepository.findMany({
          query: {
            _id: {
              $in: user.groupIds
            }
          }
        });

        groupsPermissions = groups.reduce(function (acc, group) {
          return [...acc, ...group.permissions];
        }, []);
      }

      return [...user.permissions, ...groupsPermissions];
    })();
  }

  setUserGroups({ userId, groups }) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      (0, _invariant2.default)(userId, 'User id is required!');
      (0, _invariant2.default)(groups && Array.isArray(groups), 'Groups array is required!');

      const { UserRepository, GroupRepository } = _this3;

      const user = yield UserRepository.findOne({
        query: {
          userId
        }
      });
      const fetchedGroups = yield GroupRepository.findMany({
        query: {
          name: {
            $in: groups
          }
        }
      });
      const groupIds = fetchedGroups.map(function ({ _id }) {
        return _id;
      });

      return user ? UserRepository.updateOne({
        query: {
          userId
        },
        update: {
          $set: {
            groupIds
          }
        }
      }) : UserRepository.insertOne({
        query: {
          userId,
          groupIds
        }
      });
    })();
  }
}

exports.default = Security;